/*
    Copyright (C) 2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: automation_event.h,v 1.7 2004/01/15 01:56:11 pbd Exp $
*/

#ifndef __ardour_automation_event_h__
#define __ardour_automation_event_h__

#include <list>
#include <sigc++/signal_system.h>
#include <pbd/lockmonitor.h>
#include <ardour/ardour.h>

namespace ARDOUR {
	
struct AutomationEvent {
    jack_nframes_t when;
    float value;
    
    AutomationEvent (float v, jack_nframes_t w) 
	    : when (w), value (v) { }
    
    bool operator==(const AutomationEvent& other) {
	    return value == other.value && when == other.when;
    }
};

class AutomationList : public SigC::Object
{
  public:
	typedef std::list<AutomationEvent>::iterator iterator;
	typedef std::list<AutomationEvent>::const_iterator const_iterator;

	AutomationList();
	~AutomationList() {}

	SigC::Signal0<void> Changed;

	void freeze();
	void thaw ();

	void clear ();
	void add (const AutomationEvent& ev);
	void erase_range (jack_nframes_t start, jack_nframes_t end);
	void erase (iterator);
	void erase (iterator, iterator);
	void adjust_range (iterator start, iterator end, float delta, float lower, float upper);
	void modify (iterator, jack_nframes_t, float);

	const_iterator begin() const { return _list.begin(); }
	const_iterator end() const { return _list.end(); }
	std::list<AutomationEvent>::size_type size() const { return _list.size(); }

	/* XXX this is a hack that should be removed once all automation
	   handling uses iterators.
	*/

	iterator nth (unsigned long);

	std::pair<AutomationList::iterator,AutomationList::iterator> control_points_adjacent (jack_nframes_t when);

	template<class T> void apply_to_points (T& obj, void (T::*method)(const AutomationList&)) {
		LockMonitor lm (lock, __LINE__, __FILE__);
		(obj.*method)(*this);
	}

  private:
	PBD::Lock lock;
	std::list<AutomationEvent> _list;
	bool _frozen;
	bool changed_when_thawed;

	void maybe_signal_changed ();
};

} // namespace

#endif /* __ardour_automation_event_h__ */
